# -*- cperl -*-

use Test::More;
use Data::Dumper;
BEGIN { plan tests => 23 };

use_ok("NAT::PTD");

my $ptd = new NAT::PTD("t/02_ptd.dmp");
isa_ok($ptd,"NAT::PTD");

ok(grep {$_ eq "casa"} $ptd->words);  # deve encontrar
ok(!(grep {$_ eq "albergue"} $ptd->words));  # n deve encontrar

is($ptd->count("casa"),100);
is($ptd->count("albergue"),0);

is($ptd->prob("casa","home"), .25);
is($ptd->prob("casa","house"), .25);
is($ptd->prob("casa","homens"), 0);

is_deeply([sort $ptd->trans("casa")], [qw.home house.]);
is_deeply([sort $ptd->words],[qw.casa gato.]);

## Recalculate probabilities
$ptd->reprob;
is($ptd->count("casa"),100); # count maintains
is($ptd->prob("casa","home"), .5);
is($ptd->prob("casa","house"), .5);
is($ptd->prob("casa","homens"), 0);

is($ptd->size, 250);

##
$ptd->downtr( sub {
	my ($w,$c,%t) = @_;
	$c = 0;
	toentry($w,$c,%t);
}, filter => 1);

is($ptd->count("casa"),0);

$ptd->downtr( sub { return undef}, filter => 1);
is_deeply([$ptd->words],[]);





## Reset things...
my $ptd1 = new NAT::PTD("t/02_ptd.dmp");
my $ptd2 = new NAT::PTD("t/02_ptd.dmp");
$ptd1->add($ptd2);
# adding duplicates occurrences
is($ptd1->count("casa"), 200);
is($ptd1->size, 500);
# adding with itself, maintains probabilities
is($ptd1->prob("casa","home"), .25);
is($ptd1->prob("casa","house"), .25);

$ptd1->subtractDomain("gato");
is_deeply([$ptd1->words],[qw.casa.]);





